#!/usr/bin/fades

import glob
import html
import json
import os
import subprocess
import sys
from urllib import request, parse, error

import bs4  # fades
import dateutil.parser
import pytz  # fades
from feedgen.feed import FeedGenerator  # fades

BASE_PUBLIC_URL = 'https://podcasts.taniquetil.com.ar/'
PODCAST_DIR = '/home/facundo/podcasts/'

BASE_SITE = 'https://www.mixcloud.com'
BASE_LISTING = "https://www.mixcloud.com/SuinRadio/"
SHOW_NAME = 'glosario del placer'

MONTHS = {
    'Ene': '01',
    'Feb': '02',
    'Mar': '03',
    'Abr': '04',
    'May': '05',
    'Jun': '06',
    'Jul': '07',
    'Ago': '08',
    'Sep': '09',
    'Oct': '10',
    'Nov': '11',
    'Dic': '12',
}

def find_treasure(data):
    treasures = set()
    to_find = 'previewUrl'

    def explore(data, sp=0):
        if isinstance(data, list):
            for i, item in enumerate(data):
                # print(" " * sp, "--- item", i, item)
                explore(item, sp + 1)

        elif isinstance(data, dict):
            if to_find in data:
                treasures.add(data['previewUrl'])
                return

            for key, value in data.items():
                # print(" " * sp, '--- key', key, value)
                explore(value, sp + 1)

    explore(data)
    print("===== treasure", treasures)
    paths = set()
    for t in treasures:
        paths.add(parse.urlparse(t).path)
    print("===== paths", paths)
    assert len(paths) == 1, paths
    (path,) = paths

    assert path.startswith('/previews/')
    assert path.endswith('.mp3')
    secret = path[10:-4]
    return secret


def download_audio(secret, filebase):
    p1 = "https://audio4.mixcloud.com/secure/dash2/{}.m4a".format(secret)
    p2 = "a1-x3"

    mp4path = filebase + ".mp4"
    print("Grabando", repr(mp4path))
    with open(mp4path, "wb") as fh:

        fragment = 0
        while True:
            if fragment == 0:
                url = "{}/init-{}.mp4".format(p1, p2)
            else:
                url = "{}/fragment-{:d}-{}.m4s".format(p1, fragment, p2)

            print("==== url", repr(url))
            try:
                resp = request.urlopen(url)
            except error.HTTPError as err:
                print("======= cortamos", dir(err), repr(getattr(err, 'code', '---')))
                break

            fh.write(resp.read())
            fragment += 1

    # process mp4
    mp3path = filebase + ".mp3"
    print("Codificando")
    cmd = ['ffmpeg', '-i', mp4path, '-vn', '-acodec', 'libmp3lame', '-ab', '64k', mp3path]
    subprocess.check_call(cmd)
    os.remove(mp4path)


def _get_date_from_mp3_path(filepath, showlocal_tz):
    """Parse the mp3 filepath (which we build) and retrieve the date."""
    date_str = os.path.basename(filepath).split(".")[0].split("_")[-1]
    date = dateutil.parser.parse(date_str)
    return showlocal_tz.localize(date)


def write_podcast_xml():
    fg = FeedGenerator()
    fg.load_extension('podcast')

    url = "{}gdp.xml".format(BASE_PUBLIC_URL)
    fg.id(url.split('.')[0])
    fg.title("Glosario del Placer")
    fg.image('http://podcasts.taniquetil.com.ar/gdp.jpeg')
    fg.description("Programa de Paola Kulloc")
    fg.link(href=url, rel='self')

    # collect all mp3s for the given show
    all_mp3s = glob.glob(os.path.join(PODCAST_DIR, "gdp_*.mp3"))

    showlocal_tz = pytz.timezone('America/Buenos_Aires')

    for filepath in all_mp3s:
        filename = os.path.basename(filepath)
        mp3_date = _get_date_from_mp3_path(filepath, showlocal_tz)
        mp3_size = os.stat(filepath).st_size
        mp3_url = BASE_PUBLIC_URL + filename
        mp3_id = filename.split('.')[0]
        title = "Programa del {0:%d}/{0:%m}/{0:%Y}".format(mp3_date)

        # build the rss entry
        fe = fg.add_entry()
        fe.id(mp3_id)
        fe.pubDate(mp3_date)
        fe.title(title)
        fe.enclosure(mp3_url, str(mp3_size), 'audio/mpeg')

    fg.rss_str(pretty=True)
    fg.rss_file(os.path.join(PODCAST_DIR, 'gdp.xml'))


def download(ep_date, url):
    """Download the specific episode."""
    filebase = os.path.join(PODCAST_DIR, 'gdp_' + ep_date)
    already_downloaded = glob.glob(filebase + '*')
    if already_downloaded:
        print("WARNING: ignorando episodio porque repe:", already_downloaded)
        return

    print("Downloading metadata", url)
    resp = request.urlopen(url)
    raw = resp.read()
    with open("temp-mix.html", "wb") as fh:
        fh.write(raw)
    # raw = open("temp-mix, "rb").read()
    soup = bs4.BeautifulSoup(raw, features="html.parser")

    data = soup.find('script', {'id': 'relay-data'})
    text = html.unescape(data.text)
    data = json.loads(text)

    secret_path = find_treasure(data)

    print("Downloading audio {!r} to {!r}".format(secret_path, filebase))
    download_audio(secret_path, filebase)

    write_podcast_xml()


def auto_explore():
    """Explore for new eposides and download them."""
    resp = request.urlopen(BASE_LISTING)
    raw = resp.read()
    with open("temp-mix.html", "wb") as fh:
        fh.write(raw)
    # raw = open("temp-mix, "rb").read()
    soup = bs4.BeautifulSoup(raw, features="html.parser")

    cards = soup.find_all('section', {'class': 'card cf'})
    for card in cards:
        title = card.find('h1')
        if SHOW_NAME not in title.text.lower():
            continue
        print("======= show", title)

        # get the url
        link = title.find('a')
        url = BASE_SITE + link['href']
        print("==== url", url)

        # process the date
        weird_date = title.text.split()[0]
        day, monthname, year = weird_date.split('-')
        month = MONTHS[monthname]
        ep_date = '-'.join((year, month, day))

        # download
        download(ep_date, url)


if len(sys.argv) == 2 and sys.argv[1] == '--auto':
    auto_explore()
elif len(sys.argv) == 3:
    download(sys.argv[1], sys.argv[2])
else:
    print("Usage download-mix.py <date> <program-url>")
    print("   or download-mix.py --auto")
    print("   e.g.: download-mix.py 2019-11-11 https://www.mixcloud.com/SuinRadio/11-nov-2019-gdp-glosario-del-placer-en-suin-radio-001/")  # noqa
